@@ -34,7 +34,51 @@ import (
 	"github.com/openshift/origin/test/extended/util/ibmcloud"
 )
 
-const waitForPrometheusStartSeconds = 240
+var _ = g.Describe("[Feature:Prometheus][Late] Alerts", func() {
+	defer g.GinkgoRecover()
+	var (
+		oc = exutil.NewCLIWithoutNamespace("prometheus")
+
+		url, bearerToken string
+	)
+	g.BeforeEach(func() {
+		var ok bool
+		url, bearerToken, ok = locatePrometheus(oc)
+		if !ok {
+			e2e.Failf("Prometheus could not be located on this cluster, failing prometheus test")
+		}
+	})
+	g.It("shouldn't report any alerts in firing state apart from Watchdog and AlertmanagerReceiversNotConfigured", func() {
+		if len(os.Getenv("TEST_UNSUPPORTED_ALLOW_VERSION_SKEW")) > 0 {
+			e2e.Skipf("Test is disabled to allow cluster components to have different versions, and skewed versions trigger multiple other alerts")
+		}
+		oc.SetupProject()
+		ns := oc.Namespace()
+		execPod := exutil.CreateCentosExecPodOrFail(oc.AdminKubeClient(), ns, "execpod", nil)
+		defer func() { oc.AdminKubeClient().CoreV1().Pods(ns).Delete(execPod.Name, metav1.NewDeleteOptions(1)) }()
+
+		tests := map[string]bool{
+			// Checking Watchdog alert state is done in "should have a Watchdog alert in firing state".
+			// Bug 1826033: Image pruner installs itself as suspended by default, firing an alert
+			`count_over_time(ALERTS{alertname!~"Watchdog|AlertmanagerReceiversNotConfigured|KubeAPILatencyHigh|FailingOperator|ImagePruningDisabled",alertstate="firing"}[2h]) >= 1`: false,
+		}
+		runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
+	})
+	g.It("should have a Watchdog alert in firing state the entire cluster run", func() {
+		oc.SetupProject()
+		ns := oc.Namespace()
+		execPod := exutil.CreateCentosExecPodOrFail(oc.AdminKubeClient(), ns, "execpod", nil)
+		defer func() { oc.AdminKubeClient().CoreV1().Pods(ns).Delete(execPod.Name, metav1.NewDeleteOptions(1)) }()
+
+		tests := map[string]bool{
+			// should have constantly firing a watchdog alert
+			`count_over_time(ALERTS{alertstate="firing",alertname="Watchdog", severity="none"}[1h])`: true,
+		}
+		runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
+
+		e2e.Logf("Watchdog alert is firing")
+	})
+})
 
 var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 	defer g.GinkgoRecover()
@@ -48,7 +92,7 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 		var ok bool
 		url, bearerToken, ok = locatePrometheus(oc)
 		if !ok {
-			e2e.Skipf("Prometheus could not be located on this cluster, skipping prometheus test")
+			e2e.Failf("Prometheus could not be located on this cluster, failing prometheus test")
 		}
 	})
 
@@ -64,6 +108,11 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 			defer func() { oc.AdminKubeClient().CoreV1().Pods(ns).Delete(execPod.Name, metav1.NewDeleteOptions(1)) }()
 
 			tests := map[string]bool{
+				// Should have successfully sent at least some metrics to
+				// remote write endpoint uncomment this once sending telemetry
+				// via remote write is merged, and remove the other two checks.
+				// `prometheus_remote_storage_succeeded_samples_total{job="prometheus-k8s"} >= 1`: true,
+
 				// should have successfully sent at least once to remote
 				`metricsclient_request_send{client="federate_to",job="telemeter-client",status_code="200"} >= 1`: true,
 				// should have scraped some metrics from prometheus
@@ -82,7 +131,7 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 
 			g.By("checking the unsecured metrics path")
 			var metrics map[string]*dto.MetricFamily
-			o.Expect(wait.PollImmediate(10*time.Second, waitForPrometheusStartSeconds*time.Second, func() (bool, error) {
+			o.Expect(wait.PollImmediate(10*time.Second, 2*time.Minute, func() (bool, error) {
 				results, err := getInsecureURLViaPod(ns, execPod.Name, fmt.Sprintf("%s/metrics", url))
 				if err != nil {
 					e2e.Logf("unable to get unsecured metrics: %v", err)
@@ -130,7 +179,6 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 				o.Expect(err).NotTo(o.HaveOccurred())
 
 				g.By("verifying all expected jobs have a working target")
-
 				// For IBM cloud clusters, skip control plane components and the CVO
 				if e2e.TestContext.Provider != ibmcloud.ProviderName {
 					lastErrs = all(
@@ -139,7 +187,8 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 						targets.Expect(labels{"job": "controller-manager"}, "up", "^https://.*/metrics$"),
 
 						// The kube control plane
-						targets.Expect(labels{"job": "etcd"}, "up", "^https://.*/metrics$"),
+						// TODO restore this after etcd operator lands
+						//targets.Expect(labels{"job": "etcd"}, "up", "^https://.*/metrics$"),
 						targets.Expect(labels{"job": "apiserver"}, "up", "^https://.*/metrics$"),
 						targets.Expect(labels{"job": "kube-controller-manager"}, "up", "^https://.*/metrics$"),
 						targets.Expect(labels{"job": "scheduler"}, "up", "^https://.*/metrics$"),
@@ -158,7 +207,6 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 					targets.Expect(labels{"job": "prometheus-operator"}, "up", "^http://.*/metrics$"),
 					targets.Expect(labels{"job": "alertmanager-main"}, "up", "^https://.*/metrics$"),
 					targets.Expect(labels{"job": "crio"}, "up", "^http://.*/metrics$"),
-					targets.Expect(labels{"job": "telemeter-client"}, "up", "^https://.*/metrics$"),
 				)...)
 				if len(lastErrs) > 0 {
 					e2e.Logf("missing some targets: %v", lastErrs)
@@ -167,7 +215,7 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 				return true, nil
 			})).NotTo(o.HaveOccurred(), "possibly some services didn't register ServiceMonitors to allow metrics collection")
 		})
-		g.It("should have a Watchdog alert in firing state", func() {
+		g.It("should have a AlertmanagerReceiversNotConfigured alert in firing state", func() {
 			oc.SetupProject()
 			ns := oc.Namespace()
 			execPod := exutil.CreateCentosExecPodOrFail(oc.AdminKubeClient(), ns, "execpod", nil)
@@ -175,11 +223,34 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 
 			tests := map[string]bool{
 				// should have constantly firing a watchdog alert
-				`ALERTS{alertstate="firing",alertname="Watchdog",severity="none"} == 1`: true,
+				`ALERTS{alertstate="firing",alertname="AlertmanagerReceiversNotConfigured"} == 1`: true,
 			}
 			runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
 
-			e2e.Logf("Watchdog alert is firing")
+			e2e.Logf("AlertmanagerReceiversNotConfigured alert is firing")
+		})
+		g.It("should have important platform topology metrics", func() {
+			oc.SetupProject()
+			ns := oc.Namespace()
+			execPod := exutil.CreateCentosExecPodOrFail(oc.AdminKubeClient(), ns, "execpod", nil)
+			defer func() { oc.AdminKubeClient().CoreV1().Pods(ns).Delete(execPod.Name, metav1.NewDeleteOptions(1)) }()
+
+			tests := map[string]bool{
+				// track infrastructure type
+				`cluster_infrastructure_provider{type!=""}`: true,
+				`cluster_feature_set`:                       true,
+
+				// track installer type
+				`cluster_installer{type!="",invoker!=""}`: true,
+
+				// track sum of etcd
+				`instance:etcd_object_counts:sum > 0`: true,
+
+				// track cores and sockets across node types
+				`sum(node_role_os_version_machine:cpu_capacity_cores:sum{label_kubernetes_io_arch!="",label_node_role_kubernetes_io_master!=""}) > 0`:                                      true,
+				`sum(node_role_os_version_machine:cpu_capacity_sockets:sum{label_kubernetes_io_arch!="",label_node_hyperthread_enabled!="",label_node_role_kubernetes_io_master!=""}) > 0`: true,
+			}
+			runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
 		})
 		g.It("should have non-Pod host cAdvisor metrics", func() {
 			oc.SetupProject()
@@ -217,7 +288,7 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 				runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
 			})
 		})
-		g.It("shouldn't report any alerts in firing state apart from Watchdog and AlertmanagerReceiversNotConfigured", func() {
+		g.It("shouldn't report any alerts in firing state apart from Watchdog and AlertmanagerReceiversNotConfigured [Early]", func() {
 			if len(os.Getenv("TEST_UNSUPPORTED_ALLOW_VERSION_SKEW")) > 0 {
 				e2e.Skipf("Test is disabled to allow cluster components to have different versions, and skewed versions trigger multiple other alerts")
 			}
@@ -228,8 +299,8 @@ var _ = g.Describe("[Feature:Prometheus][Conformance] Prometheus", func() {
 
 			tests := map[string]bool{
 				// Checking Watchdog alert state is done in "should have a Watchdog alert in firing state".
-				// FIXME(paulfantom): UsingDeprecatedAPIExtensionsV1Beta1 shouldn't be firing!
-				`ALERTS{alertname!~"Watchdog|AlertmanagerReceiversNotConfigured|UsingDeprecatedAPIExtensionsV1Beta1",alertstate="firing"} >= 1`: false,
+				// Bug 1826033: Image pruner installs itself as suspended by default, firing an alert
+				`ALERTS{alertname!~"Watchdog|AlertmanagerReceiversNotConfigured|PrometheusRemoteWriteDesiredShards|ImagePruningDisabled",alertstate="firing"} >= 1`: false,
 			}
 			runQueries(tests, oc, ns, execPod.Name, url, bearerToken)
 		})
